# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from telnyx.types.porting_orders import (
    PhoneNumberConfigurationListResponse,
    PhoneNumberConfigurationCreateResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPhoneNumberConfigurations:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        phone_number_configuration = client.porting_orders.phone_number_configurations.create()
        assert_matches_type(PhoneNumberConfigurationCreateResponse, phone_number_configuration, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        phone_number_configuration = client.porting_orders.phone_number_configurations.create(
            phone_number_configurations=[
                {
                    "porting_phone_number_id": "927f4687-318c-44bc-9f2f-22a5898143a4",
                    "user_bundle_id": "ff901545-3e27-462a-ba9d-2b34654cab82",
                }
            ],
        )
        assert_matches_type(PhoneNumberConfigurationCreateResponse, phone_number_configuration, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.porting_orders.phone_number_configurations.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_configuration = response.parse()
        assert_matches_type(PhoneNumberConfigurationCreateResponse, phone_number_configuration, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.porting_orders.phone_number_configurations.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_configuration = response.parse()
            assert_matches_type(PhoneNumberConfigurationCreateResponse, phone_number_configuration, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        phone_number_configuration = client.porting_orders.phone_number_configurations.list()
        assert_matches_type(
            SyncDefaultFlatPagination[PhoneNumberConfigurationListResponse],
            phone_number_configuration,
            path=["response"],
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        phone_number_configuration = client.porting_orders.phone_number_configurations.list(
            filter={
                "porting_order": {"status": ["in-process"]},
                "porting_phone_number": ["5d6f7ede-1961-4717-bfb5-db392c5efc2d"],
                "user_bundle_id": ["5d6f7ede-1961-4717-bfb5-db392c5efc2d"],
            },
            page_number=0,
            page_size=0,
            sort={"value": "created_at"},
        )
        assert_matches_type(
            SyncDefaultFlatPagination[PhoneNumberConfigurationListResponse],
            phone_number_configuration,
            path=["response"],
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.porting_orders.phone_number_configurations.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_configuration = response.parse()
        assert_matches_type(
            SyncDefaultFlatPagination[PhoneNumberConfigurationListResponse],
            phone_number_configuration,
            path=["response"],
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.porting_orders.phone_number_configurations.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_configuration = response.parse()
            assert_matches_type(
                SyncDefaultFlatPagination[PhoneNumberConfigurationListResponse],
                phone_number_configuration,
                path=["response"],
            )

        assert cast(Any, response.is_closed) is True


class TestAsyncPhoneNumberConfigurations:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        phone_number_configuration = await async_client.porting_orders.phone_number_configurations.create()
        assert_matches_type(PhoneNumberConfigurationCreateResponse, phone_number_configuration, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        phone_number_configuration = await async_client.porting_orders.phone_number_configurations.create(
            phone_number_configurations=[
                {
                    "porting_phone_number_id": "927f4687-318c-44bc-9f2f-22a5898143a4",
                    "user_bundle_id": "ff901545-3e27-462a-ba9d-2b34654cab82",
                }
            ],
        )
        assert_matches_type(PhoneNumberConfigurationCreateResponse, phone_number_configuration, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.porting_orders.phone_number_configurations.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_configuration = await response.parse()
        assert_matches_type(PhoneNumberConfigurationCreateResponse, phone_number_configuration, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.porting_orders.phone_number_configurations.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_configuration = await response.parse()
            assert_matches_type(PhoneNumberConfigurationCreateResponse, phone_number_configuration, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        phone_number_configuration = await async_client.porting_orders.phone_number_configurations.list()
        assert_matches_type(
            AsyncDefaultFlatPagination[PhoneNumberConfigurationListResponse],
            phone_number_configuration,
            path=["response"],
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        phone_number_configuration = await async_client.porting_orders.phone_number_configurations.list(
            filter={
                "porting_order": {"status": ["in-process"]},
                "porting_phone_number": ["5d6f7ede-1961-4717-bfb5-db392c5efc2d"],
                "user_bundle_id": ["5d6f7ede-1961-4717-bfb5-db392c5efc2d"],
            },
            page_number=0,
            page_size=0,
            sort={"value": "created_at"},
        )
        assert_matches_type(
            AsyncDefaultFlatPagination[PhoneNumberConfigurationListResponse],
            phone_number_configuration,
            path=["response"],
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.porting_orders.phone_number_configurations.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_configuration = await response.parse()
        assert_matches_type(
            AsyncDefaultFlatPagination[PhoneNumberConfigurationListResponse],
            phone_number_configuration,
            path=["response"],
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.porting_orders.phone_number_configurations.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_configuration = await response.parse()
            assert_matches_type(
                AsyncDefaultFlatPagination[PhoneNumberConfigurationListResponse],
                phone_number_configuration,
                path=["response"],
            )

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from ..._types import FileTypes

__all__ = ["ActionUploadFileParams"]


class ActionUploadFileParams(TypedDict, total=False):
    bill: FileTypes
    """
    Must be the last month's bill with proof of ownership of all of the numbers in
    the order in PDF format.
    """

    loa: FileTypes
    """Must be a signed LOA for the numbers in the order in PDF format."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .verify_profile import VerifyProfile

__all__ = ["VerifyProfileData"]


class VerifyProfileData(BaseModel):
    data: Optional[VerifyProfile] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .call_siprec_stopped import CallSiprecStopped

__all__ = ["CallSiprecStoppedWebhookEvent"]


class CallSiprecStoppedWebhookEvent(BaseModel):
    data: Optional[CallSiprecStopped] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = ["BillingBundleListParams", "Filter"]


class BillingBundleListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Supports filtering by country_iso and resource. Examples: filter[country_iso]=US
    or filter[resource]=+15617819942
    """

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]

    authorization_bearer: str
    """Authenticates the request with your Telnyx API V2 KEY"""


class Filter(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style).

    Supports filtering by country_iso and resource. Examples: filter[country_iso]=US or filter[resource]=+15617819942
    """

    country_iso: SequenceNotStr[str]
    """Filter by country code."""

    resource: SequenceNotStr[str]
    """Filter by resource."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["BucketCreatePresignedURLParams"]


class BucketCreatePresignedURLParams(TypedDict, total=False):
    bucket_name: Required[Annotated[str, PropertyInfo(alias="bucketName")]]

    ttl: int
    """The time to live of the token in seconds"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .verified_number import VerifiedNumber

__all__ = ["VerifiedNumberDataWrapper"]


class VerifiedNumberDataWrapper(BaseModel):
    data: Optional[VerifiedNumber] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["SimCardOrderPreviewPreviewResponse", "Data", "DataShippingCost", "DataSimCardsCost", "DataTotalCost"]


class DataShippingCost(BaseModel):
    amount: Optional[str] = None
    """A string representing the cost amount."""

    currency: Optional[str] = None
    """ISO 4217 currency string."""


class DataSimCardsCost(BaseModel):
    amount: Optional[str] = None
    """A string representing the cost amount."""

    currency: Optional[str] = None
    """ISO 4217 currency string."""


class DataTotalCost(BaseModel):
    amount: Optional[str] = None
    """A string representing the cost amount."""

    currency: Optional[str] = None
    """ISO 4217 currency string."""


class Data(BaseModel):
    quantity: Optional[int] = None
    """The amount of SIM cards requested in the SIM card order."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    shipping_cost: Optional[DataShippingCost] = None

    sim_cards_cost: Optional[DataSimCardsCost] = None

    total_cost: Optional[DataTotalCost] = None


class SimCardOrderPreviewPreviewResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["InexplicitNumberOrderListParams"]


class InexplicitNumberOrderListParams(TypedDict, total=False):
    page_number: int
    """The page number to load"""

    page_size: int
    """The size of the page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    MessageSendResponse,
    MessageRetrieveResponse,
    MessageScheduleResponse,
    MessageSendGroupMmsResponse,
    MessageSendLongCodeResponse,
    MessageSendWhatsappResponse,
    MessageSendShortCodeResponse,
    MessageSendNumberPoolResponse,
    MessageCancelScheduledResponse,
)
from telnyx._utils import parse_datetime

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestMessages:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        message = client.messages.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(MessageRetrieveResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.messages.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = response.parse()
        assert_matches_type(MessageRetrieveResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.messages.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            message = response.parse()
            assert_matches_type(MessageRetrieveResponse, message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.messages.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_cancel_scheduled(self, client: Telnyx) -> None:
        message = client.messages.cancel_scheduled(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(MessageCancelScheduledResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_cancel_scheduled(self, client: Telnyx) -> None:
        response = client.messages.with_raw_response.cancel_scheduled(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = response.parse()
        assert_matches_type(MessageCancelScheduledResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_cancel_scheduled(self, client: Telnyx) -> None:
        with client.messages.with_streaming_response.cancel_scheduled(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            message = response.parse()
            assert_matches_type(MessageCancelScheduledResponse, message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_cancel_scheduled(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.messages.with_raw_response.cancel_scheduled(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_schedule(self, client: Telnyx) -> None:
        message = client.messages.schedule(
            to="+18445550001",
        )
        assert_matches_type(MessageScheduleResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_schedule_with_all_params(self, client: Telnyx) -> None:
        message = client.messages.schedule(
            to="+18445550001",
            auto_detect=True,
            from_="+18445550001",
            media_urls=["string"],
            messaging_profile_id="abc85f64-5717-4562-b3fc-2c9600000000",
            send_at=parse_datetime("2019-01-23T18:30:00Z"),
            subject="From Telnyx!",
            text="Hello, World!",
            type="SMS",
            use_profile_webhooks=True,
            webhook_failover_url="https://backup.example.com/hooks",
            webhook_url="http://example.com/webhooks",
        )
        assert_matches_type(MessageScheduleResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_schedule(self, client: Telnyx) -> None:
        response = client.messages.with_raw_response.schedule(
            to="+18445550001",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = response.parse()
        assert_matches_type(MessageScheduleResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_schedule(self, client: Telnyx) -> None:
        with client.messages.with_streaming_response.schedule(
            to="+18445550001",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            message = response.parse()
            assert_matches_type(MessageScheduleResponse, message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_send(self, client: Telnyx) -> None:
        message = client.messages.send(
            to="+18445550001",
        )
        assert_matches_type(MessageSendResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_send_with_all_params(self, client: Telnyx) -> None:
        message = client.messages.send(
            to="+18445550001",
            auto_detect=True,
            encoding="auto",
            from_="+18445550001",
            media_urls=["http://example.com"],
            messaging_profile_id="abc85f64-5717-4562-b3fc-2c9600000000",
            send_at=parse_datetime("2019-12-27T18:11:19.117Z"),
            subject="From Telnyx!",
            text="Hello, World!",
            type="MMS",
            use_profile_webhooks=True,
            webhook_failover_url="https://backup.example.com/hooks",
            webhook_url="http://example.com/webhooks",
        )
        assert_matches_type(MessageSendResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_send(self, client: Telnyx) -> None:
        response = client.messages.with_raw_response.send(
            to="+18445550001",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = response.parse()
        assert_matches_type(MessageSendResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_send(self, client: Telnyx) -> None:
        with client.messages.with_streaming_response.send(
            to="+18445550001",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            message = response.parse()
            assert_matches_type(MessageSendResponse, message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_send_group_mms(self, client: Telnyx) -> None:
        message = client.messages.send_group_mms(
            from_="+13125551234",
            to=["+18655551234", "+14155551234"],
        )
        assert_matches_type(MessageSendGroupMmsResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_send_group_mms_with_all_params(self, client: Telnyx) -> None:
        message = client.messages.send_group_mms(
            from_="+13125551234",
            to=["+18655551234", "+14155551234"],
            media_urls=["http://example.com"],
            subject="From Telnyx!",
            text="Hello, World!",
            use_profile_webhooks=True,
            webhook_failover_url="https://backup.example.com/hooks",
            webhook_url="http://example.com/webhooks",
        )
        assert_matches_type(MessageSendGroupMmsResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_send_group_mms(self, client: Telnyx) -> None:
        response = client.messages.with_raw_response.send_group_mms(
            from_="+13125551234",
            to=["+18655551234", "+14155551234"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = response.parse()
        assert_matches_type(MessageSendGroupMmsResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_send_group_mms(self, client: Telnyx) -> None:
        with client.messages.with_streaming_response.send_group_mms(
            from_="+13125551234",
            to=["+18655551234", "+14155551234"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            message = response.parse()
            assert_matches_type(MessageSendGroupMmsResponse, message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_send_long_code(self, client: Telnyx) -> None:
        message = client.messages.send_long_code(
            from_="+18445550001",
            to="+13125550002",
        )
        assert_matches_type(MessageSendLongCodeResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_send_long_code_with_all_params(self, client: Telnyx) -> None:
        message = client.messages.send_long_code(
            from_="+18445550001",
            to="+13125550002",
            auto_detect=True,
            encoding="auto",
            media_urls=["http://example.com"],
            subject="From Telnyx!",
            text="Hello, World!",
            type="MMS",
            use_profile_webhooks=True,
            webhook_failover_url="https://backup.example.com/hooks",
            webhook_url="http://example.com/webhooks",
        )
        assert_matches_type(MessageSendLongCodeResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_send_long_code(self, client: Telnyx) -> None:
        response = client.messages.with_raw_response.send_long_code(
            from_="+18445550001",
            to="+13125550002",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = response.parse()
        assert_matches_type(MessageSendLongCodeResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_send_long_code(self, client: Telnyx) -> None:
        with client.messages.with_streaming_response.send_long_code(
            from_="+18445550001",
            to="+13125550002",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            message = response.parse()
            assert_matches_type(MessageSendLongCodeResponse, message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_send_number_pool(self, client: Telnyx) -> None:
        message = client.messages.send_number_pool(
            messaging_profile_id="abc85f64-5717-4562-b3fc-2c9600000000",
            to="+13125550002",
        )
        assert_matches_type(MessageSendNumberPoolResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_send_number_pool_with_all_params(self, client: Telnyx) -> None:
        message = client.messages.send_number_pool(
            messaging_profile_id="abc85f64-5717-4562-b3fc-2c9600000000",
            to="+13125550002",
            auto_detect=True,
            encoding="auto",
            media_urls=["http://example.com"],
            subject="From Telnyx!",
            text="Hello, World!",
            type="MMS",
            use_profile_webhooks=True,
            webhook_failover_url="https://backup.example.com/hooks",
            webhook_url="http://example.com/webhooks",
        )
        assert_matches_type(MessageSendNumberPoolResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_send_number_pool(self, client: Telnyx) -> None:
        response = client.messages.with_raw_response.send_number_pool(
            messaging_profile_id="abc85f64-5717-4562-b3fc-2c9600000000",
            to="+13125550002",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = response.parse()
        assert_matches_type(MessageSendNumberPoolResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_send_number_pool(self, client: Telnyx) -> None:
        with client.messages.with_streaming_response.send_number_pool(
            messaging_profile_id="abc85f64-5717-4562-b3fc-2c9600000000",
            to="+13125550002",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            message = response.parse()
            assert_matches_type(MessageSendNumberPoolResponse, message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_send_short_code(self, client: Telnyx) -> None:
        message = client.messages.send_short_code(
            from_="+18445550001",
            to="+18445550001",
        )
        assert_matches_type(MessageSendShortCodeResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_send_short_code_with_all_params(self, client: Telnyx) -> None:
        message = client.messages.send_short_code(
            from_="+18445550001",
            to="+18445550001",
            auto_detect=True,
            encoding="auto",
            media_urls=["http://example.com"],
            subject="From Telnyx!",
            text="Hello, World!",
            type="MMS",
            use_profile_webhooks=True,
            webhook_failover_url="https://backup.example.com/hooks",
            webhook_url="http://example.com/webhooks",
        )
        assert_matches_type(MessageSendShortCodeResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_send_short_code(self, client: Telnyx) -> None:
        response = client.messages.with_raw_response.send_short_code(
            from_="+18445550001",
            to="+18445550001",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = response.parse()
        assert_matches_type(MessageSendShortCodeResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_send_short_code(self, client: Telnyx) -> None:
        with client.messages.with_streaming_response.send_short_code(
            from_="+18445550001",
            to="+18445550001",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            message = response.parse()
            assert_matches_type(MessageSendShortCodeResponse, message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_send_whatsapp(self, client: Telnyx) -> None:
        message = client.messages.send_whatsapp(
            from_="+13125551234",
            to="+13125551234",
            whatsapp_message={},
        )
        assert_matches_type(MessageSendWhatsappResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_send_whatsapp_with_all_params(self, client: Telnyx) -> None:
        message = client.messages.send_whatsapp(
            from_="+13125551234",
            to="+13125551234",
            whatsapp_message={
                "audio": {
                    "caption": "caption",
                    "filename": "filename",
                    "link": "http://example.com/media.jpg",
                    "voice": True,
                },
                "biz_opaque_callback_data": "biz_opaque_callback_data",
                "contacts": [
                    {
                        "addresses": [
                            {
                                "city": "city",
                                "country": "country",
                                "country_code": "country_code",
                                "state": "state",
                                "street": "street",
                                "type": "type",
                                "zip": "zip",
                            }
                        ],
                        "birthday": "birthday",
                        "emails": [
                            {
                                "email": "email",
                                "type": "type",
                            }
                        ],
                        "name": "name",
                        "org": {
                            "company": "company",
                            "department": "department",
                            "title": "title",
                        },
                        "phones": [
                            {
                                "phone": "phone",
                                "type": "type",
                                "wa_id": "wa_id",
                            }
                        ],
                        "urls": [
                            {
                                "type": "type",
                                "url": "url",
                            }
                        ],
                    }
                ],
                "document": {
                    "caption": "caption",
                    "filename": "filename",
                    "link": "http://example.com/media.jpg",
                    "voice": True,
                },
                "image": {
                    "caption": "caption",
                    "filename": "filename",
                    "link": "http://example.com/media.jpg",
                    "voice": True,
                },
                "interactive": {
                    "action": {
                        "button": "button",
                        "buttons": [
                            {
                                "reply": {
                                    "id": "id",
                                    "title": "title",
                                },
                                "type": "reply",
                            }
                        ],
                        "cards": [
                            {
                                "action": {
                                    "catalog_id": "catalog_id",
                                    "product_retailer_id": "product_retailer_id",
                                },
                                "body": {"text": "text"},
                                "card_index": 0,
                                "header": {
                                    "image": {
                                        "caption": "caption",
                                        "filename": "filename",
                                        "link": "http://example.com/media.jpg",
                                        "voice": True,
                                    },
                                    "type": "image",
                                    "video": {
                                        "caption": "caption",
                                        "filename": "filename",
                                        "link": "http://example.com/media.jpg",
                                        "voice": True,
                                    },
                                },
                                "type": "cta_url",
                            }
                        ],
                        "catalog_id": "catalog_id",
                        "mode": "mode",
                        "name": "name",
                        "parameters": {
                            "display_text": "display_text",
                            "url": "url",
                        },
                        "product_retailer_id": "product_retailer_id",
                        "sections": [
                            {
                                "product_items": [{"product_retailer_id": "product_retailer_id"}],
                                "rows": [
                                    {
                                        "id": "id",
                                        "description": "description",
                                        "title": "title",
                                    }
                                ],
                                "title": "title",
                            }
                        ],
                    },
                    "body": {"text": "text"},
                    "footer": {"text": "text"},
                    "header": {
                        "document": {
                            "caption": "caption",
                            "filename": "filename",
                            "link": "http://example.com/media.jpg",
                            "voice": True,
                        },
                        "image": {
                            "caption": "caption",
                            "filename": "filename",
                            "link": "http://example.com/media.jpg",
                            "voice": True,
                        },
                        "sub_text": "sub_text",
                        "text": "text",
                        "video": {
                            "caption": "caption",
                            "filename": "filename",
                            "link": "http://example.com/media.jpg",
                            "voice": True,
                        },
                    },
                    "type": "cta_url",
                },
                "location": {
                    "address": "address",
                    "latitude": "latitude",
                    "longitude": "longitude",
                    "name": "name",
                },
                "reaction": {
                    "emoji": "emoji",
                    "message_id": "message_id",
                },
                "sticker": {
                    "caption": "caption",
                    "filename": "filename",
                    "link": "http://example.com/media.jpg",
                    "voice": True,
                },
                "type": "audio",
                "video": {
                    "caption": "caption",
                    "filename": "filename",
                    "link": "http://example.com/media.jpg",
                    "voice": True,
                },
            },
            type="WHATSAPP",
            webhook_url="webhook_url",
        )
        assert_matches_type(MessageSendWhatsappResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_send_whatsapp(self, client: Telnyx) -> None:
        response = client.messages.with_raw_response.send_whatsapp(
            from_="+13125551234",
            to="+13125551234",
            whatsapp_message={},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = response.parse()
        assert_matches_type(MessageSendWhatsappResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_send_whatsapp(self, client: Telnyx) -> None:
        with client.messages.with_streaming_response.send_whatsapp(
            from_="+13125551234",
            to="+13125551234",
            whatsapp_message={},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            message = response.parse()
            assert_matches_type(MessageSendWhatsappResponse, message, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncMessages:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        message = await async_client.messages.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(MessageRetrieveResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messages.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = await response.parse()
        assert_matches_type(MessageRetrieveResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messages.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            message = await response.parse()
            assert_matches_type(MessageRetrieveResponse, message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.messages.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_cancel_scheduled(self, async_client: AsyncTelnyx) -> None:
        message = await async_client.messages.cancel_scheduled(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(MessageCancelScheduledResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_cancel_scheduled(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messages.with_raw_response.cancel_scheduled(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = await response.parse()
        assert_matches_type(MessageCancelScheduledResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_cancel_scheduled(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messages.with_streaming_response.cancel_scheduled(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            message = await response.parse()
            assert_matches_type(MessageCancelScheduledResponse, message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_cancel_scheduled(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.messages.with_raw_response.cancel_scheduled(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_schedule(self, async_client: AsyncTelnyx) -> None:
        message = await async_client.messages.schedule(
            to="+18445550001",
        )
        assert_matches_type(MessageScheduleResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_schedule_with_all_params(self, async_client: AsyncTelnyx) -> None:
        message = await async_client.messages.schedule(
            to="+18445550001",
            auto_detect=True,
            from_="+18445550001",
            media_urls=["string"],
            messaging_profile_id="abc85f64-5717-4562-b3fc-2c9600000000",
            send_at=parse_datetime("2019-01-23T18:30:00Z"),
            subject="From Telnyx!",
            text="Hello, World!",
            type="SMS",
            use_profile_webhooks=True,
            webhook_failover_url="https://backup.example.com/hooks",
            webhook_url="http://example.com/webhooks",
        )
        assert_matches_type(MessageScheduleResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_schedule(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messages.with_raw_response.schedule(
            to="+18445550001",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = await response.parse()
        assert_matches_type(MessageScheduleResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_schedule(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messages.with_streaming_response.schedule(
            to="+18445550001",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            message = await response.parse()
            assert_matches_type(MessageScheduleResponse, message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_send(self, async_client: AsyncTelnyx) -> None:
        message = await async_client.messages.send(
            to="+18445550001",
        )
        assert_matches_type(MessageSendResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_send_with_all_params(self, async_client: AsyncTelnyx) -> None:
        message = await async_client.messages.send(
            to="+18445550001",
            auto_detect=True,
            encoding="auto",
            from_="+18445550001",
            media_urls=["http://example.com"],
            messaging_profile_id="abc85f64-5717-4562-b3fc-2c9600000000",
            send_at=parse_datetime("2019-12-27T18:11:19.117Z"),
            subject="From Telnyx!",
            text="Hello, World!",
            type="MMS",
            use_profile_webhooks=True,
            webhook_failover_url="https://backup.example.com/hooks",
            webhook_url="http://example.com/webhooks",
        )
        assert_matches_type(MessageSendResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_send(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messages.with_raw_response.send(
            to="+18445550001",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = await response.parse()
        assert_matches_type(MessageSendResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_send(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messages.with_streaming_response.send(
            to="+18445550001",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            message = await response.parse()
            assert_matches_type(MessageSendResponse, message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_send_group_mms(self, async_client: AsyncTelnyx) -> None:
        message = await async_client.messages.send_group_mms(
            from_="+13125551234",
            to=["+18655551234", "+14155551234"],
        )
        assert_matches_type(MessageSendGroupMmsResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_send_group_mms_with_all_params(self, async_client: AsyncTelnyx) -> None:
        message = await async_client.messages.send_group_mms(
            from_="+13125551234",
            to=["+18655551234", "+14155551234"],
            media_urls=["http://example.com"],
            subject="From Telnyx!",
            text="Hello, World!",
            use_profile_webhooks=True,
            webhook_failover_url="https://backup.example.com/hooks",
            webhook_url="http://example.com/webhooks",
        )
        assert_matches_type(MessageSendGroupMmsResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_send_group_mms(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messages.with_raw_response.send_group_mms(
            from_="+13125551234",
            to=["+18655551234", "+14155551234"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = await response.parse()
        assert_matches_type(MessageSendGroupMmsResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_send_group_mms(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messages.with_streaming_response.send_group_mms(
            from_="+13125551234",
            to=["+18655551234", "+14155551234"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            message = await response.parse()
            assert_matches_type(MessageSendGroupMmsResponse, message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_send_long_code(self, async_client: AsyncTelnyx) -> None:
        message = await async_client.messages.send_long_code(
            from_="+18445550001",
            to="+13125550002",
        )
        assert_matches_type(MessageSendLongCodeResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_send_long_code_with_all_params(self, async_client: AsyncTelnyx) -> None:
        message = await async_client.messages.send_long_code(
            from_="+18445550001",
            to="+13125550002",
            auto_detect=True,
            encoding="auto",
            media_urls=["http://example.com"],
            subject="From Telnyx!",
            text="Hello, World!",
            type="MMS",
            use_profile_webhooks=True,
            webhook_failover_url="https://backup.example.com/hooks",
            webhook_url="http://example.com/webhooks",
        )
        assert_matches_type(MessageSendLongCodeResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_send_long_code(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messages.with_raw_response.send_long_code(
            from_="+18445550001",
            to="+13125550002",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = await response.parse()
        assert_matches_type(MessageSendLongCodeResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_send_long_code(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messages.with_streaming_response.send_long_code(
            from_="+18445550001",
            to="+13125550002",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            message = await response.parse()
            assert_matches_type(MessageSendLongCodeResponse, message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_send_number_pool(self, async_client: AsyncTelnyx) -> None:
        message = await async_client.messages.send_number_pool(
            messaging_profile_id="abc85f64-5717-4562-b3fc-2c9600000000",
            to="+13125550002",
        )
        assert_matches_type(MessageSendNumberPoolResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_send_number_pool_with_all_params(self, async_client: AsyncTelnyx) -> None:
        message = await async_client.messages.send_number_pool(
            messaging_profile_id="abc85f64-5717-4562-b3fc-2c9600000000",
            to="+13125550002",
            auto_detect=True,
            encoding="auto",
            media_urls=["http://example.com"],
            subject="From Telnyx!",
            text="Hello, World!",
            type="MMS",
            use_profile_webhooks=True,
            webhook_failover_url="https://backup.example.com/hooks",
            webhook_url="http://example.com/webhooks",
        )
        assert_matches_type(MessageSendNumberPoolResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_send_number_pool(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messages.with_raw_response.send_number_pool(
            messaging_profile_id="abc85f64-5717-4562-b3fc-2c9600000000",
            to="+13125550002",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = await response.parse()
        assert_matches_type(MessageSendNumberPoolResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_send_number_pool(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messages.with_streaming_response.send_number_pool(
            messaging_profile_id="abc85f64-5717-4562-b3fc-2c9600000000",
            to="+13125550002",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            message = await response.parse()
            assert_matches_type(MessageSendNumberPoolResponse, message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_send_short_code(self, async_client: AsyncTelnyx) -> None:
        message = await async_client.messages.send_short_code(
            from_="+18445550001",
            to="+18445550001",
        )
        assert_matches_type(MessageSendShortCodeResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_send_short_code_with_all_params(self, async_client: AsyncTelnyx) -> None:
        message = await async_client.messages.send_short_code(
            from_="+18445550001",
            to="+18445550001",
            auto_detect=True,
            encoding="auto",
            media_urls=["http://example.com"],
            subject="From Telnyx!",
            text="Hello, World!",
            type="MMS",
            use_profile_webhooks=True,
            webhook_failover_url="https://backup.example.com/hooks",
            webhook_url="http://example.com/webhooks",
        )
        assert_matches_type(MessageSendShortCodeResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_send_short_code(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messages.with_raw_response.send_short_code(
            from_="+18445550001",
            to="+18445550001",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = await response.parse()
        assert_matches_type(MessageSendShortCodeResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_send_short_code(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messages.with_streaming_response.send_short_code(
            from_="+18445550001",
            to="+18445550001",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            message = await response.parse()
            assert_matches_type(MessageSendShortCodeResponse, message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_send_whatsapp(self, async_client: AsyncTelnyx) -> None:
        message = await async_client.messages.send_whatsapp(
            from_="+13125551234",
            to="+13125551234",
            whatsapp_message={},
        )
        assert_matches_type(MessageSendWhatsappResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_send_whatsapp_with_all_params(self, async_client: AsyncTelnyx) -> None:
        message = await async_client.messages.send_whatsapp(
            from_="+13125551234",
            to="+13125551234",
            whatsapp_message={
                "audio": {
                    "caption": "caption",
                    "filename": "filename",
                    "link": "http://example.com/media.jpg",
                    "voice": True,
                },
                "biz_opaque_callback_data": "biz_opaque_callback_data",
                "contacts": [
                    {
                        "addresses": [
                            {
                                "city": "city",
                                "country": "country",
                                "country_code": "country_code",
                                "state": "state",
                                "street": "street",
                                "type": "type",
                                "zip": "zip",
                            }
                        ],
                        "birthday": "birthday",
                        "emails": [
                            {
                                "email": "email",
                                "type": "type",
                            }
                        ],
                        "name": "name",
                        "org": {
                            "company": "company",
                            "department": "department",
                            "title": "title",
                        },
                        "phones": [
                            {
                                "phone": "phone",
                                "type": "type",
                                "wa_id": "wa_id",
                            }
                        ],
                        "urls": [
                            {
                                "type": "type",
                                "url": "url",
                            }
                        ],
                    }
                ],
                "document": {
                    "caption": "caption",
                    "filename": "filename",
                    "link": "http://example.com/media.jpg",
                    "voice": True,
                },
                "image": {
                    "caption": "caption",
                    "filename": "filename",
                    "link": "http://example.com/media.jpg",
                    "voice": True,
                },
                "interactive": {
                    "action": {
                        "button": "button",
                        "buttons": [
                            {
                                "reply": {
                                    "id": "id",
                                    "title": "title",
                                },
                                "type": "reply",
                            }
                        ],
                        "cards": [
                            {
                                "action": {
                                    "catalog_id": "catalog_id",
                                    "product_retailer_id": "product_retailer_id",
                                },
                                "body": {"text": "text"},
                                "card_index": 0,
                                "header": {
                                    "image": {
                                        "caption": "caption",
                                        "filename": "filename",
                                        "link": "http://example.com/media.jpg",
                                        "voice": True,
                                    },
                                    "type": "image",
                                    "video": {
                                        "caption": "caption",
                                        "filename": "filename",
                                        "link": "http://example.com/media.jpg",
                                        "voice": True,
                                    },
                                },
                                "type": "cta_url",
                            }
                        ],
                        "catalog_id": "catalog_id",
                        "mode": "mode",
                        "name": "name",
                        "parameters": {
                            "display_text": "display_text",
                            "url": "url",
                        },
                        "product_retailer_id": "product_retailer_id",
                        "sections": [
                            {
                                "product_items": [{"product_retailer_id": "product_retailer_id"}],
                                "rows": [
                                    {
                                        "id": "id",
                                        "description": "description",
                                        "title": "title",
                                    }
                                ],
                                "title": "title",
                            }
                        ],
                    },
                    "body": {"text": "text"},
                    "footer": {"text": "text"},
                    "header": {
                        "document": {
                            "caption": "caption",
                            "filename": "filename",
                            "link": "http://example.com/media.jpg",
                            "voice": True,
                        },
                        "image": {
                            "caption": "caption",
                            "filename": "filename",
                            "link": "http://example.com/media.jpg",
                            "voice": True,
                        },
                        "sub_text": "sub_text",
                        "text": "text",
                        "video": {
                            "caption": "caption",
                            "filename": "filename",
                            "link": "http://example.com/media.jpg",
                            "voice": True,
                        },
                    },
                    "type": "cta_url",
                },
                "location": {
                    "address": "address",
                    "latitude": "latitude",
                    "longitude": "longitude",
                    "name": "name",
                },
                "reaction": {
                    "emoji": "emoji",
                    "message_id": "message_id",
                },
                "sticker": {
                    "caption": "caption",
                    "filename": "filename",
                    "link": "http://example.com/media.jpg",
                    "voice": True,
                },
                "type": "audio",
                "video": {
                    "caption": "caption",
                    "filename": "filename",
                    "link": "http://example.com/media.jpg",
                    "voice": True,
                },
            },
            type="WHATSAPP",
            webhook_url="webhook_url",
        )
        assert_matches_type(MessageSendWhatsappResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_send_whatsapp(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messages.with_raw_response.send_whatsapp(
            from_="+13125551234",
            to="+13125551234",
            whatsapp_message={},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = await response.parse()
        assert_matches_type(MessageSendWhatsappResponse, message, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_send_whatsapp(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messages.with_streaming_response.send_whatsapp(
            from_="+13125551234",
            to="+13125551234",
            whatsapp_message={},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            message = await response.parse()
            assert_matches_type(MessageSendWhatsappResponse, message, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Annotated, TypeAlias, TypedDict

from .._utils import PropertyInfo

__all__ = [
    "VirtualCrossConnectsCoverageListParams",
    "Filter",
    "Filters",
    "FiltersAvailableBandwidth",
    "FiltersAvailableBandwidthContains",
]


class VirtualCrossConnectsCoverageListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[cloud_provider], filter[cloud_provider_region],
    filter[location.region], filter[location.site], filter[location.pop],
    filter[location.code]
    """

    filters: Filters
    """Consolidated filters parameter (deepObject style).

    Originally: filters[available_bandwidth][contains]
    """

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]


class Filter(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[cloud_provider], filter[cloud_provider_region], filter[location.region], filter[location.site], filter[location.pop], filter[location.code]
    """

    cloud_provider: Literal["aws", "azure", "gce"]
    """The Virtual Private Cloud provider."""

    cloud_provider_region: str
    """The region of specific cloud provider."""

    location_code: Annotated[str, PropertyInfo(alias="location.code")]
    """The code of associated location to filter on."""

    location_pop: Annotated[str, PropertyInfo(alias="location.pop")]
    """The POP of associated location to filter on."""

    location_region: Annotated[str, PropertyInfo(alias="location.region")]
    """The region of associated location to filter on."""

    location_site: Annotated[str, PropertyInfo(alias="location.site")]
    """The site of associated location to filter on."""


class FiltersAvailableBandwidthContains(TypedDict, total=False):
    """Available bandwidth filtering operations"""

    contains: int
    """Filter by available bandwidth containing the specified value"""


FiltersAvailableBandwidth: TypeAlias = Union[int, FiltersAvailableBandwidthContains]


class Filters(TypedDict, total=False):
    """Consolidated filters parameter (deepObject style).

    Originally: filters[available_bandwidth][contains]
    """

    available_bandwidth: FiltersAvailableBandwidth
    """Filter by exact available bandwidth match"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .plan_step_data import PlanStepData as PlanStepData
from .event_log_params import EventLogParams as EventLogParams
from .event_list_params import EventListParams as EventListParams
from .event_log_response import EventLogResponse as EventLogResponse
from .plan_create_params import PlanCreateParams as PlanCreateParams
from .event_list_response import EventListResponse as EventListResponse
from .plan_create_response import PlanCreateResponse as PlanCreateResponse
from .plan_retrieve_response import PlanRetrieveResponse as PlanRetrieveResponse
from .plan_update_step_params import PlanUpdateStepParams as PlanUpdateStepParams
from .telnyx_agent_link_params import TelnyxAgentLinkParams as TelnyxAgentLinkParams
from .plan_update_step_response import PlanUpdateStepResponse as PlanUpdateStepResponse
from .telnyx_agent_link_response import TelnyxAgentLinkResponse as TelnyxAgentLinkResponse
from .telnyx_agent_list_response import TelnyxAgentListResponse as TelnyxAgentListResponse
from .plan_add_steps_to_plan_params import PlanAddStepsToPlanParams as PlanAddStepsToPlanParams
from .plan_get_step_details_response import PlanGetStepDetailsResponse as PlanGetStepDetailsResponse
from .plan_add_steps_to_plan_response import PlanAddStepsToPlanResponse as PlanAddStepsToPlanResponse
from .event_get_event_details_response import EventGetEventDetailsResponse as EventGetEventDetailsResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    to_custom_raw_response_wrapper,
    async_to_streamed_response_wrapper,
    to_custom_streamed_response_wrapper,
    async_to_custom_raw_response_wrapper,
    async_to_custom_streamed_response_wrapper,
)
from ...pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.porting import (
    loa_configuration_list_params,
    loa_configuration_create_params,
    loa_configuration_update_params,
    loa_configuration_preview_0_params,
)
from ...types.porting.porting_loa_configuration import PortingLoaConfiguration
from ...types.porting.loa_configuration_create_response import LoaConfigurationCreateResponse
from ...types.porting.loa_configuration_update_response import LoaConfigurationUpdateResponse
from ...types.porting.loa_configuration_retrieve_response import LoaConfigurationRetrieveResponse

__all__ = ["LoaConfigurationsResource", "AsyncLoaConfigurationsResource"]


class LoaConfigurationsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> LoaConfigurationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return LoaConfigurationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LoaConfigurationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return LoaConfigurationsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        address: loa_configuration_create_params.Address,
        company_name: str,
        contact: loa_configuration_create_params.Contact,
        logo: loa_configuration_create_params.Logo,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LoaConfigurationCreateResponse:
        """
        Create a LOA configuration.

        Args:
          address: The address of the company.

          company_name: The name of the company

          contact: The contact information of the company.

          logo: The logo of the LOA configuration

          name: The name of the LOA configuration

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/porting/loa_configurations",
            body=maybe_transform(
                {
                    "address": address,
                    "company_name": company_name,
                    "contact": contact,
                    "logo": logo,
                    "name": name,
                },
                loa_configuration_create_params.LoaConfigurationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LoaConfigurationCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LoaConfigurationRetrieveResponse:
        """
        Retrieve a specific LOA configuration.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/porting/loa_configurations/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LoaConfigurationRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        address: loa_configuration_update_params.Address,
        company_name: str,
        contact: loa_configuration_update_params.Contact,
        logo: loa_configuration_update_params.Logo,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LoaConfigurationUpdateResponse:
        """
        Update a specific LOA configuration.

        Args:
          address: The address of the company.

          company_name: The name of the company

          contact: The contact information of the company.

          logo: The logo of the LOA configuration

          name: The name of the LOA configuration

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/porting/loa_configurations/{id}",
            body=maybe_transform(
                {
                    "address": address,
                    "company_name": company_name,
                    "contact": contact,
                    "logo": logo,
                    "name": name,
                },
                loa_configuration_update_params.LoaConfigurationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LoaConfigurationUpdateResponse,
        )

    def list(
        self,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[PortingLoaConfiguration]:
        """
        List the LOA configurations.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/porting/loa_configurations",
            page=SyncDefaultFlatPagination[PortingLoaConfiguration],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    loa_configuration_list_params.LoaConfigurationListParams,
                ),
            ),
            model=PortingLoaConfiguration,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a specific LOA configuration.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/porting/loa_configurations/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def preview_0(
        self,
        *,
        address: loa_configuration_preview_0_params.Address,
        company_name: str,
        contact: loa_configuration_preview_0_params.Contact,
        logo: loa_configuration_preview_0_params.Logo,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BinaryAPIResponse:
        """
        Preview the LOA template that would be generated without need to create LOA
        configuration.

        Args:
          address: The address of the company.

          company_name: The name of the company

          contact: The contact information of the company.

          logo: The logo of the LOA configuration

          name: The name of the LOA configuration

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "application/pdf", **(extra_headers or {})}
        return self._post(
            "/porting/loa_configuration/preview",
            body=maybe_transform(
                {
                    "address": address,
                    "company_name": company_name,
                    "contact": contact,
                    "logo": logo,
                    "name": name,
                },
                loa_configuration_preview_0_params.LoaConfigurationPreview0Params,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BinaryAPIResponse,
        )

    def preview_1(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BinaryAPIResponse:
        """
        Preview a specific LOA configuration.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/pdf", **(extra_headers or {})}
        return self._get(
            f"/porting/loa_configurations/{id}/preview",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BinaryAPIResponse,
        )


class AsyncLoaConfigurationsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncLoaConfigurationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLoaConfigurationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLoaConfigurationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncLoaConfigurationsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        address: loa_configuration_create_params.Address,
        company_name: str,
        contact: loa_configuration_create_params.Contact,
        logo: loa_configuration_create_params.Logo,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LoaConfigurationCreateResponse:
        """
        Create a LOA configuration.

        Args:
          address: The address of the company.

          company_name: The name of the company

          contact: The contact information of the company.

          logo: The logo of the LOA configuration

          name: The name of the LOA configuration

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/porting/loa_configurations",
            body=await async_maybe_transform(
                {
                    "address": address,
                    "company_name": company_name,
                    "contact": contact,
                    "logo": logo,
                    "name": name,
                },
                loa_configuration_create_params.LoaConfigurationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LoaConfigurationCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LoaConfigurationRetrieveResponse:
        """
        Retrieve a specific LOA configuration.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/porting/loa_configurations/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LoaConfigurationRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        address: loa_configuration_update_params.Address,
        company_name: str,
        contact: loa_configuration_update_params.Contact,
        logo: loa_configuration_update_params.Logo,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LoaConfigurationUpdateResponse:
        """
        Update a specific LOA configuration.

        Args:
          address: The address of the company.

          company_name: The name of the company

          contact: The contact information of the company.

          logo: The logo of the LOA configuration

          name: The name of the LOA configuration

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/porting/loa_configurations/{id}",
            body=await async_maybe_transform(
                {
                    "address": address,
                    "company_name": company_name,
                    "contact": contact,
                    "logo": logo,
                    "name": name,
                },
                loa_configuration_update_params.LoaConfigurationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LoaConfigurationUpdateResponse,
        )

    def list(
        self,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[PortingLoaConfiguration, AsyncDefaultFlatPagination[PortingLoaConfiguration]]:
        """
        List the LOA configurations.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/porting/loa_configurations",
            page=AsyncDefaultFlatPagination[PortingLoaConfiguration],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    loa_configuration_list_params.LoaConfigurationListParams,
                ),
            ),
            model=PortingLoaConfiguration,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a specific LOA configuration.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/porting/loa_configurations/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def preview_0(
        self,
        *,
        address: loa_configuration_preview_0_params.Address,
        company_name: str,
        contact: loa_configuration_preview_0_params.Contact,
        logo: loa_configuration_preview_0_params.Logo,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncBinaryAPIResponse:
        """
        Preview the LOA template that would be generated without need to create LOA
        configuration.

        Args:
          address: The address of the company.

          company_name: The name of the company

          contact: The contact information of the company.

          logo: The logo of the LOA configuration

          name: The name of the LOA configuration

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "application/pdf", **(extra_headers or {})}
        return await self._post(
            "/porting/loa_configuration/preview",
            body=await async_maybe_transform(
                {
                    "address": address,
                    "company_name": company_name,
                    "contact": contact,
                    "logo": logo,
                    "name": name,
                },
                loa_configuration_preview_0_params.LoaConfigurationPreview0Params,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AsyncBinaryAPIResponse,
        )

    async def preview_1(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncBinaryAPIResponse:
        """
        Preview a specific LOA configuration.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/pdf", **(extra_headers or {})}
        return await self._get(
            f"/porting/loa_configurations/{id}/preview",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AsyncBinaryAPIResponse,
        )


class LoaConfigurationsResourceWithRawResponse:
    def __init__(self, loa_configurations: LoaConfigurationsResource) -> None:
        self._loa_configurations = loa_configurations

        self.create = to_raw_response_wrapper(
            loa_configurations.create,
        )
        self.retrieve = to_raw_response_wrapper(
            loa_configurations.retrieve,
        )
        self.update = to_raw_response_wrapper(
            loa_configurations.update,
        )
        self.list = to_raw_response_wrapper(
            loa_configurations.list,
        )
        self.delete = to_raw_response_wrapper(
            loa_configurations.delete,
        )
        self.preview_0 = to_custom_raw_response_wrapper(
            loa_configurations.preview_0,
            BinaryAPIResponse,
        )
        self.preview_1 = to_custom_raw_response_wrapper(
            loa_configurations.preview_1,
            BinaryAPIResponse,
        )


class AsyncLoaConfigurationsResourceWithRawResponse:
    def __init__(self, loa_configurations: AsyncLoaConfigurationsResource) -> None:
        self._loa_configurations = loa_configurations

        self.create = async_to_raw_response_wrapper(
            loa_configurations.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            loa_configurations.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            loa_configurations.update,
        )
        self.list = async_to_raw_response_wrapper(
            loa_configurations.list,
        )
        self.delete = async_to_raw_response_wrapper(
            loa_configurations.delete,
        )
        self.preview_0 = async_to_custom_raw_response_wrapper(
            loa_configurations.preview_0,
            AsyncBinaryAPIResponse,
        )
        self.preview_1 = async_to_custom_raw_response_wrapper(
            loa_configurations.preview_1,
            AsyncBinaryAPIResponse,
        )


class LoaConfigurationsResourceWithStreamingResponse:
    def __init__(self, loa_configurations: LoaConfigurationsResource) -> None:
        self._loa_configurations = loa_configurations

        self.create = to_streamed_response_wrapper(
            loa_configurations.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            loa_configurations.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            loa_configurations.update,
        )
        self.list = to_streamed_response_wrapper(
            loa_configurations.list,
        )
        self.delete = to_streamed_response_wrapper(
            loa_configurations.delete,
        )
        self.preview_0 = to_custom_streamed_response_wrapper(
            loa_configurations.preview_0,
            StreamedBinaryAPIResponse,
        )
        self.preview_1 = to_custom_streamed_response_wrapper(
            loa_configurations.preview_1,
            StreamedBinaryAPIResponse,
        )


class AsyncLoaConfigurationsResourceWithStreamingResponse:
    def __init__(self, loa_configurations: AsyncLoaConfigurationsResource) -> None:
        self._loa_configurations = loa_configurations

        self.create = async_to_streamed_response_wrapper(
            loa_configurations.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            loa_configurations.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            loa_configurations.update,
        )
        self.list = async_to_streamed_response_wrapper(
            loa_configurations.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            loa_configurations.delete,
        )
        self.preview_0 = async_to_custom_streamed_response_wrapper(
            loa_configurations.preview_0,
            AsyncStreamedBinaryAPIResponse,
        )
        self.preview_1 = async_to_custom_streamed_response_wrapper(
            loa_configurations.preview_1,
            AsyncStreamedBinaryAPIResponse,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from .mdr_usage_report import MdrUsageReport

__all__ = ["MdrUsageReportRetrieveResponse"]


class MdrUsageReportRetrieveResponse(BaseModel):
    data: Optional[MdrUsageReport] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["McpServerUpdateResponse"]


class McpServerUpdateResponse(BaseModel):
    id: str

    created_at: datetime

    name: str

    type: str

    url: str

    allowed_tools: Optional[List[str]] = None

    api_key_ref: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["NumberOrderListResponse", "PhoneNumber"]


class PhoneNumber(BaseModel):
    """The unique phone numbers given as arguments in the job creation."""

    id: Optional[str] = None
    """The phone number's ID"""

    phone_number: Optional[str] = None
    """The phone number in e164 format."""


class NumberOrderListResponse(BaseModel):
    id: Optional[str] = None

    billing_group_id: Optional[str] = None
    """Identifies the messaging profile associated with the phone number."""

    connection_id: Optional[str] = None
    """Identifies the connection associated with this phone number."""

    created_at: Optional[datetime] = None
    """An ISO 8901 datetime string denoting when the number order was created."""

    customer_reference: Optional[str] = None
    """A customer reference string for customer look ups."""

    messaging_profile_id: Optional[str] = None
    """Identifies the messaging profile associated with the phone number."""

    phone_numbers: Optional[List[PhoneNumber]] = None

    phone_numbers_count: Optional[int] = None
    """The count of phone numbers in the number order."""

    record_type: Optional[str] = None

    requirements_met: Optional[bool] = None
    """True if all requirements are met for every phone number, false otherwise."""

    status: Optional[Literal["pending", "success", "failure"]] = None
    """The status of the order."""

    sub_number_orders_ids: Optional[List[str]] = None

    updated_at: Optional[datetime] = None
    """An ISO 8901 datetime string for when the number order was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    ReportListMdrsResponse,
    ReportListWdrsResponse,
)
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestReports:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_mdrs(self, client: Telnyx) -> None:
        report = client.reports.list_mdrs()
        assert_matches_type(ReportListMdrsResponse, report, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_mdrs_with_all_params(self, client: Telnyx) -> None:
        report = client.reports.list_mdrs(
            id="e093fbe0-5bde-11eb-ae93-0242ac130002",
            cld="+15551237654",
            cli="+15551237654",
            direction="INBOUND",
            end_date="end_date",
            message_type="SMS",
            profile="My profile",
            start_date="start_date",
            status="DELIVERED",
        )
        assert_matches_type(ReportListMdrsResponse, report, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list_mdrs(self, client: Telnyx) -> None:
        response = client.reports.with_raw_response.list_mdrs()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        report = response.parse()
        assert_matches_type(ReportListMdrsResponse, report, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list_mdrs(self, client: Telnyx) -> None:
        with client.reports.with_streaming_response.list_mdrs() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            report = response.parse()
            assert_matches_type(ReportListMdrsResponse, report, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_wdrs(self, client: Telnyx) -> None:
        report = client.reports.list_wdrs()
        assert_matches_type(SyncDefaultFlatPagination[ReportListWdrsResponse], report, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_wdrs_with_all_params(self, client: Telnyx) -> None:
        report = client.reports.list_wdrs(
            id="e093fbe0-5bde-11eb-ae93-0242ac130002",
            end_date="2021-06-01T00:00:00Z",
            imsi="123456",
            mcc="204",
            mnc="01",
            page_number=0,
            page_size=0,
            phone_number="+12345678910",
            sim_card_id="877f80a6-e5b2-4687-9a04-88076265720f",
            sim_group_id="f05a189f-7c46-4531-ac56-1460dc465a42",
            sim_group_name="sim name",
            sort=["created_at"],
            start_date="2021-05-01T00:00:00Z",
        )
        assert_matches_type(SyncDefaultFlatPagination[ReportListWdrsResponse], report, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list_wdrs(self, client: Telnyx) -> None:
        response = client.reports.with_raw_response.list_wdrs()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        report = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[ReportListWdrsResponse], report, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list_wdrs(self, client: Telnyx) -> None:
        with client.reports.with_streaming_response.list_wdrs() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            report = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[ReportListWdrsResponse], report, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncReports:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_mdrs(self, async_client: AsyncTelnyx) -> None:
        report = await async_client.reports.list_mdrs()
        assert_matches_type(ReportListMdrsResponse, report, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_mdrs_with_all_params(self, async_client: AsyncTelnyx) -> None:
        report = await async_client.reports.list_mdrs(
            id="e093fbe0-5bde-11eb-ae93-0242ac130002",
            cld="+15551237654",
            cli="+15551237654",
            direction="INBOUND",
            end_date="end_date",
            message_type="SMS",
            profile="My profile",
            start_date="start_date",
            status="DELIVERED",
        )
        assert_matches_type(ReportListMdrsResponse, report, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list_mdrs(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.reports.with_raw_response.list_mdrs()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        report = await response.parse()
        assert_matches_type(ReportListMdrsResponse, report, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list_mdrs(self, async_client: AsyncTelnyx) -> None:
        async with async_client.reports.with_streaming_response.list_mdrs() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            report = await response.parse()
            assert_matches_type(ReportListMdrsResponse, report, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_wdrs(self, async_client: AsyncTelnyx) -> None:
        report = await async_client.reports.list_wdrs()
        assert_matches_type(AsyncDefaultFlatPagination[ReportListWdrsResponse], report, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_wdrs_with_all_params(self, async_client: AsyncTelnyx) -> None:
        report = await async_client.reports.list_wdrs(
            id="e093fbe0-5bde-11eb-ae93-0242ac130002",
            end_date="2021-06-01T00:00:00Z",
            imsi="123456",
            mcc="204",
            mnc="01",
            page_number=0,
            page_size=0,
            phone_number="+12345678910",
            sim_card_id="877f80a6-e5b2-4687-9a04-88076265720f",
            sim_group_id="f05a189f-7c46-4531-ac56-1460dc465a42",
            sim_group_name="sim name",
            sort=["created_at"],
            start_date="2021-05-01T00:00:00Z",
        )
        assert_matches_type(AsyncDefaultFlatPagination[ReportListWdrsResponse], report, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list_wdrs(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.reports.with_raw_response.list_wdrs()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        report = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[ReportListWdrsResponse], report, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list_wdrs(self, async_client: AsyncTelnyx) -> None:
        async with async_client.reports.with_streaming_response.list_wdrs() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            report = await response.parse()
            assert_matches_type(AsyncDefaultFlatPagination[ReportListWdrsResponse], report, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    CallControlApplication,
    CallControlApplicationCreateResponse,
    CallControlApplicationDeleteResponse,
    CallControlApplicationUpdateResponse,
    CallControlApplicationRetrieveResponse,
)
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestCallControlApplications:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        call_control_application = client.call_control_applications.create(
            application_name="call-router",
            webhook_event_url="https://example.com",
        )
        assert_matches_type(CallControlApplicationCreateResponse, call_control_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        call_control_application = client.call_control_applications.create(
            application_name="call-router",
            webhook_event_url="https://example.com",
            active=False,
            anchorsite_override="Latency",
            call_cost_in_webhooks=True,
            dtmf_type="Inband",
            first_command_timeout=True,
            first_command_timeout_secs=10,
            inbound={
                "channel_limit": 10,
                "shaken_stir_enabled": True,
                "sip_subdomain": "example",
                "sip_subdomain_receive_settings": "only_my_connections",
            },
            outbound={
                "channel_limit": 10,
                "outbound_voice_profile_id": "1293384261075731499",
            },
            redact_dtmf_debug_logging=True,
            webhook_api_version="1",
            webhook_event_failover_url="https://failover.example.com",
            webhook_timeout_secs=25,
        )
        assert_matches_type(CallControlApplicationCreateResponse, call_control_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.call_control_applications.with_raw_response.create(
            application_name="call-router",
            webhook_event_url="https://example.com",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        call_control_application = response.parse()
        assert_matches_type(CallControlApplicationCreateResponse, call_control_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.call_control_applications.with_streaming_response.create(
            application_name="call-router",
            webhook_event_url="https://example.com",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            call_control_application = response.parse()
            assert_matches_type(CallControlApplicationCreateResponse, call_control_application, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        call_control_application = client.call_control_applications.retrieve(
            "id",
        )
        assert_matches_type(CallControlApplicationRetrieveResponse, call_control_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.call_control_applications.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        call_control_application = response.parse()
        assert_matches_type(CallControlApplicationRetrieveResponse, call_control_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.call_control_applications.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            call_control_application = response.parse()
            assert_matches_type(CallControlApplicationRetrieveResponse, call_control_application, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.call_control_applications.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        call_control_application = client.call_control_applications.update(
            id="id",
            application_name="call-router",
            webhook_event_url="https://example.com",
        )
        assert_matches_type(CallControlApplicationUpdateResponse, call_control_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        call_control_application = client.call_control_applications.update(
            id="id",
            application_name="call-router",
            webhook_event_url="https://example.com",
            active=False,
            anchorsite_override="Latency",
            call_cost_in_webhooks=True,
            dtmf_type="Inband",
            first_command_timeout=True,
            first_command_timeout_secs=10,
            inbound={
                "channel_limit": 10,
                "shaken_stir_enabled": True,
                "sip_subdomain": "example",
                "sip_subdomain_receive_settings": "only_my_connections",
            },
            outbound={
                "channel_limit": 10,
                "outbound_voice_profile_id": "1293384261075731499",
            },
            redact_dtmf_debug_logging=True,
            tags=["tag1", "tag2"],
            webhook_api_version="1",
            webhook_event_failover_url="https://failover.example.com",
            webhook_timeout_secs=25,
        )
        assert_matches_type(CallControlApplicationUpdateResponse, call_control_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.call_control_applications.with_raw_response.update(
            id="id",
            application_name="call-router",
            webhook_event_url="https://example.com",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        call_control_application = response.parse()
        assert_matches_type(CallControlApplicationUpdateResponse, call_control_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.call_control_applications.with_streaming_response.update(
            id="id",
            application_name="call-router",
            webhook_event_url="https://example.com",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            call_control_application = response.parse()
            assert_matches_type(CallControlApplicationUpdateResponse, call_control_application, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.call_control_applications.with_raw_response.update(
                id="",
                application_name="call-router",
                webhook_event_url="https://example.com",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        call_control_application = client.call_control_applications.list()
        assert_matches_type(
            SyncDefaultFlatPagination[CallControlApplication], call_control_application, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        call_control_application = client.call_control_applications.list(
            filter={
                "application_name": {"contains": "contains"},
                "application_session_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                "connection_id": "connection_id",
                "failed": False,
                "from": "+12025550142",
                "leg_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                "name": "name",
                "occurred_at": {
                    "eq": "2019-03-29T11:10:00Z",
                    "gt": "2019-03-29T11:10:00Z",
                    "gte": "2019-03-29T11:10:00Z",
                    "lt": "2019-03-29T11:10:00Z",
                    "lte": "2019-03-29T11:10:00Z",
                },
                "outbound_outbound_voice_profile_id": "1293384261075731499",
                "product": "texml",
                "status": "init",
                "to": "+12025550142",
                "type": "webhook",
            },
            page_number=0,
            page_size=0,
            sort="connection_name",
        )
        assert_matches_type(
            SyncDefaultFlatPagination[CallControlApplication], call_control_application, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.call_control_applications.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        call_control_application = response.parse()
        assert_matches_type(
            SyncDefaultFlatPagination[CallControlApplication], call_control_application, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.call_control_applications.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            call_control_application = response.parse()
            assert_matches_type(
                SyncDefaultFlatPagination[CallControlApplication], call_control_application, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        call_control_application = client.call_control_applications.delete(
            "id",
        )
        assert_matches_type(CallControlApplicationDeleteResponse, call_control_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.call_control_applications.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        call_control_application = response.parse()
        assert_matches_type(CallControlApplicationDeleteResponse, call_control_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.call_control_applications.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            call_control_application = response.parse()
            assert_matches_type(CallControlApplicationDeleteResponse, call_control_application, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.call_control_applications.with_raw_response.delete(
                "",
            )


class TestAsyncCallControlApplications:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        call_control_application = await async_client.call_control_applications.create(
            application_name="call-router",
            webhook_event_url="https://example.com",
        )
        assert_matches_type(CallControlApplicationCreateResponse, call_control_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        call_control_application = await async_client.call_control_applications.create(
            application_name="call-router",
            webhook_event_url="https://example.com",
            active=False,
            anchorsite_override="Latency",
            call_cost_in_webhooks=True,
            dtmf_type="Inband",
            first_command_timeout=True,
            first_command_timeout_secs=10,
            inbound={
                "channel_limit": 10,
                "shaken_stir_enabled": True,
                "sip_subdomain": "example",
                "sip_subdomain_receive_settings": "only_my_connections",
            },
            outbound={
                "channel_limit": 10,
                "outbound_voice_profile_id": "1293384261075731499",
            },
            redact_dtmf_debug_logging=True,
            webhook_api_version="1",
            webhook_event_failover_url="https://failover.example.com",
            webhook_timeout_secs=25,
        )
        assert_matches_type(CallControlApplicationCreateResponse, call_control_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.call_control_applications.with_raw_response.create(
            application_name="call-router",
            webhook_event_url="https://example.com",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        call_control_application = await response.parse()
        assert_matches_type(CallControlApplicationCreateResponse, call_control_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.call_control_applications.with_streaming_response.create(
            application_name="call-router",
            webhook_event_url="https://example.com",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            call_control_application = await response.parse()
            assert_matches_type(CallControlApplicationCreateResponse, call_control_application, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        call_control_application = await async_client.call_control_applications.retrieve(
            "id",
        )
        assert_matches_type(CallControlApplicationRetrieveResponse, call_control_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.call_control_applications.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        call_control_application = await response.parse()
        assert_matches_type(CallControlApplicationRetrieveResponse, call_control_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.call_control_applications.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            call_control_application = await response.parse()
            assert_matches_type(CallControlApplicationRetrieveResponse, call_control_application, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.call_control_applications.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        call_control_application = await async_client.call_control_applications.update(
            id="id",
            application_name="call-router",
            webhook_event_url="https://example.com",
        )
        assert_matches_type(CallControlApplicationUpdateResponse, call_control_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        call_control_application = await async_client.call_control_applications.update(
            id="id",
            application_name="call-router",
            webhook_event_url="https://example.com",
            active=False,
            anchorsite_override="Latency",
            call_cost_in_webhooks=True,
            dtmf_type="Inband",
            first_command_timeout=True,
            first_command_timeout_secs=10,
            inbound={
                "channel_limit": 10,
                "shaken_stir_enabled": True,
                "sip_subdomain": "example",
                "sip_subdomain_receive_settings": "only_my_connections",
            },
            outbound={
                "channel_limit": 10,
                "outbound_voice_profile_id": "1293384261075731499",
            },
            redact_dtmf_debug_logging=True,
            tags=["tag1", "tag2"],
            webhook_api_version="1",
            webhook_event_failover_url="https://failover.example.com",
            webhook_timeout_secs=25,
        )
        assert_matches_type(CallControlApplicationUpdateResponse, call_control_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.call_control_applications.with_raw_response.update(
            id="id",
            application_name="call-router",
            webhook_event_url="https://example.com",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        call_control_application = await response.parse()
        assert_matches_type(CallControlApplicationUpdateResponse, call_control_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.call_control_applications.with_streaming_response.update(
            id="id",
            application_name="call-router",
            webhook_event_url="https://example.com",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            call_control_application = await response.parse()
            assert_matches_type(CallControlApplicationUpdateResponse, call_control_application, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.call_control_applications.with_raw_response.update(
                id="",
                application_name="call-router",
                webhook_event_url="https://example.com",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        call_control_application = await async_client.call_control_applications.list()
        assert_matches_type(
            AsyncDefaultFlatPagination[CallControlApplication], call_control_application, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        call_control_application = await async_client.call_control_applications.list(
            filter={
                "application_name": {"contains": "contains"},
                "application_session_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                "connection_id": "connection_id",
                "failed": False,
                "from": "+12025550142",
                "leg_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                "name": "name",
                "occurred_at": {
                    "eq": "2019-03-29T11:10:00Z",
                    "gt": "2019-03-29T11:10:00Z",
                    "gte": "2019-03-29T11:10:00Z",
                    "lt": "2019-03-29T11:10:00Z",
                    "lte": "2019-03-29T11:10:00Z",
                },
                "outbound_outbound_voice_profile_id": "1293384261075731499",
                "product": "texml",
                "status": "init",
                "to": "+12025550142",
                "type": "webhook",
            },
            page_number=0,
            page_size=0,
            sort="connection_name",
        )
        assert_matches_type(
            AsyncDefaultFlatPagination[CallControlApplication], call_control_application, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.call_control_applications.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        call_control_application = await response.parse()
        assert_matches_type(
            AsyncDefaultFlatPagination[CallControlApplication], call_control_application, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.call_control_applications.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            call_control_application = await response.parse()
            assert_matches_type(
                AsyncDefaultFlatPagination[CallControlApplication], call_control_application, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        call_control_application = await async_client.call_control_applications.delete(
            "id",
        )
        assert_matches_type(CallControlApplicationDeleteResponse, call_control_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.call_control_applications.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        call_control_application = await response.parse()
        assert_matches_type(CallControlApplicationDeleteResponse, call_control_application, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.call_control_applications.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            call_control_application = await response.parse()
            assert_matches_type(CallControlApplicationDeleteResponse, call_control_application, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.call_control_applications.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .usage import (
    UsageResource,
    AsyncUsageResource,
    UsageResourceWithRawResponse,
    AsyncUsageResourceWithRawResponse,
    UsageResourceWithStreamingResponse,
    AsyncUsageResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from .ssl_certificate import (
    SslCertificateResource,
    AsyncSslCertificateResource,
    SslCertificateResourceWithRawResponse,
    AsyncSslCertificateResourceWithRawResponse,
    SslCertificateResourceWithStreamingResponse,
    AsyncSslCertificateResourceWithStreamingResponse,
)
from ....types.storage import bucket_create_presigned_url_params
from ....types.storage.bucket_create_presigned_url_response import BucketCreatePresignedURLResponse

__all__ = ["BucketsResource", "AsyncBucketsResource"]


class BucketsResource(SyncAPIResource):
    @cached_property
    def ssl_certificate(self) -> SslCertificateResource:
        return SslCertificateResource(self._client)

    @cached_property
    def usage(self) -> UsageResource:
        return UsageResource(self._client)

    @cached_property
    def with_raw_response(self) -> BucketsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return BucketsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BucketsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return BucketsResourceWithStreamingResponse(self)

    def create_presigned_url(
        self,
        object_name: str,
        *,
        bucket_name: str,
        ttl: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BucketCreatePresignedURLResponse:
        """
        Returns a timed and authenticated URL to download (GET) or upload (PUT) an
        object. This is the equivalent to AWS S3’s “presigned” URL. Please note that
        Telnyx performs authentication differently from AWS S3 and you MUST NOT use the
        presign method of AWS s3api CLI or SDK to generate the presigned URL.

        Refer to: https://developers.telnyx.com/docs/cloud-storage/presigned-urls

        Args:
          ttl: The time to live of the token in seconds

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        if not object_name:
            raise ValueError(f"Expected a non-empty value for `object_name` but received {object_name!r}")
        return self._post(
            f"/storage/buckets/{bucket_name}/{object_name}/presigned_url",
            body=maybe_transform({"ttl": ttl}, bucket_create_presigned_url_params.BucketCreatePresignedURLParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BucketCreatePresignedURLResponse,
        )


class AsyncBucketsResource(AsyncAPIResource):
    @cached_property
    def ssl_certificate(self) -> AsyncSslCertificateResource:
        return AsyncSslCertificateResource(self._client)

    @cached_property
    def usage(self) -> AsyncUsageResource:
        return AsyncUsageResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncBucketsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBucketsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBucketsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncBucketsResourceWithStreamingResponse(self)

    async def create_presigned_url(
        self,
        object_name: str,
        *,
        bucket_name: str,
        ttl: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BucketCreatePresignedURLResponse:
        """
        Returns a timed and authenticated URL to download (GET) or upload (PUT) an
        object. This is the equivalent to AWS S3’s “presigned” URL. Please note that
        Telnyx performs authentication differently from AWS S3 and you MUST NOT use the
        presign method of AWS s3api CLI or SDK to generate the presigned URL.

        Refer to: https://developers.telnyx.com/docs/cloud-storage/presigned-urls

        Args:
          ttl: The time to live of the token in seconds

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        if not object_name:
            raise ValueError(f"Expected a non-empty value for `object_name` but received {object_name!r}")
        return await self._post(
            f"/storage/buckets/{bucket_name}/{object_name}/presigned_url",
            body=await async_maybe_transform(
                {"ttl": ttl}, bucket_create_presigned_url_params.BucketCreatePresignedURLParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BucketCreatePresignedURLResponse,
        )


class BucketsResourceWithRawResponse:
    def __init__(self, buckets: BucketsResource) -> None:
        self._buckets = buckets

        self.create_presigned_url = to_raw_response_wrapper(
            buckets.create_presigned_url,
        )

    @cached_property
    def ssl_certificate(self) -> SslCertificateResourceWithRawResponse:
        return SslCertificateResourceWithRawResponse(self._buckets.ssl_certificate)

    @cached_property
    def usage(self) -> UsageResourceWithRawResponse:
        return UsageResourceWithRawResponse(self._buckets.usage)


class AsyncBucketsResourceWithRawResponse:
    def __init__(self, buckets: AsyncBucketsResource) -> None:
        self._buckets = buckets

        self.create_presigned_url = async_to_raw_response_wrapper(
            buckets.create_presigned_url,
        )

    @cached_property
    def ssl_certificate(self) -> AsyncSslCertificateResourceWithRawResponse:
        return AsyncSslCertificateResourceWithRawResponse(self._buckets.ssl_certificate)

    @cached_property
    def usage(self) -> AsyncUsageResourceWithRawResponse:
        return AsyncUsageResourceWithRawResponse(self._buckets.usage)


class BucketsResourceWithStreamingResponse:
    def __init__(self, buckets: BucketsResource) -> None:
        self._buckets = buckets

        self.create_presigned_url = to_streamed_response_wrapper(
            buckets.create_presigned_url,
        )

    @cached_property
    def ssl_certificate(self) -> SslCertificateResourceWithStreamingResponse:
        return SslCertificateResourceWithStreamingResponse(self._buckets.ssl_certificate)

    @cached_property
    def usage(self) -> UsageResourceWithStreamingResponse:
        return UsageResourceWithStreamingResponse(self._buckets.usage)


class AsyncBucketsResourceWithStreamingResponse:
    def __init__(self, buckets: AsyncBucketsResource) -> None:
        self._buckets = buckets

        self.create_presigned_url = async_to_streamed_response_wrapper(
            buckets.create_presigned_url,
        )

    @cached_property
    def ssl_certificate(self) -> AsyncSslCertificateResourceWithStreamingResponse:
        return AsyncSslCertificateResourceWithStreamingResponse(self._buckets.ssl_certificate)

    @cached_property
    def usage(self) -> AsyncUsageResourceWithStreamingResponse:
        return AsyncUsageResourceWithStreamingResponse(self._buckets.usage)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["OperatorConnectResource", "AsyncOperatorConnectResource"]


class OperatorConnectResource(SyncAPIResource):
    @cached_property
    def actions(self) -> ActionsResource:
        return ActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> OperatorConnectResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return OperatorConnectResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OperatorConnectResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return OperatorConnectResourceWithStreamingResponse(self)


class AsyncOperatorConnectResource(AsyncAPIResource):
    @cached_property
    def actions(self) -> AsyncActionsResource:
        return AsyncActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncOperatorConnectResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncOperatorConnectResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOperatorConnectResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncOperatorConnectResourceWithStreamingResponse(self)


class OperatorConnectResourceWithRawResponse:
    def __init__(self, operator_connect: OperatorConnectResource) -> None:
        self._operator_connect = operator_connect

    @cached_property
    def actions(self) -> ActionsResourceWithRawResponse:
        return ActionsResourceWithRawResponse(self._operator_connect.actions)


class AsyncOperatorConnectResourceWithRawResponse:
    def __init__(self, operator_connect: AsyncOperatorConnectResource) -> None:
        self._operator_connect = operator_connect

    @cached_property
    def actions(self) -> AsyncActionsResourceWithRawResponse:
        return AsyncActionsResourceWithRawResponse(self._operator_connect.actions)


class OperatorConnectResourceWithStreamingResponse:
    def __init__(self, operator_connect: OperatorConnectResource) -> None:
        self._operator_connect = operator_connect

    @cached_property
    def actions(self) -> ActionsResourceWithStreamingResponse:
        return ActionsResourceWithStreamingResponse(self._operator_connect.actions)


class AsyncOperatorConnectResourceWithStreamingResponse:
    def __init__(self, operator_connect: AsyncOperatorConnectResource) -> None:
        self._operator_connect = operator_connect

    @cached_property
    def actions(self) -> AsyncActionsResourceWithStreamingResponse:
        return AsyncActionsResourceWithStreamingResponse(self._operator_connect.actions)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ActionEnqueueParams"]


class ActionEnqueueParams(TypedDict, total=False):
    queue_name: Required[str]
    """The name of the queue the call should be put in.

    If a queue with a given name doesn't exist yet it will be created.
    """

    client_state: str
    """Use this field to add state to every subsequent webhook.

    It must be a valid Base-64 encoded string.
    """

    command_id: str
    """Use this field to avoid duplicate commands.

    Telnyx will ignore any command with the same `command_id` for the same
    `call_control_id`.
    """

    keep_after_hangup: bool
    """If set to true, the call will remain in the queue after hangup.

    In this case bridging to such call will fail with necessary information needed
    to re-establish the call.
    """

    max_size: int
    """The maximum number of calls allowed in the queue at a given time.

    Can't be modified for an existing queue.
    """

    max_wait_time_secs: int
    """The number of seconds after which the call will be removed from the queue."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["OAuthClientListParams"]


class OAuthClientListParams(TypedDict, total=False):
    filter_allowed_grant_types_contains: Annotated[
        Literal["client_credentials", "authorization_code", "refresh_token"],
        PropertyInfo(alias="filter[allowed_grant_types][contains]"),
    ]
    """Filter by allowed grant type"""

    filter_client_id: Annotated[str, PropertyInfo(alias="filter[client_id]")]
    """Filter by client ID"""

    filter_client_type: Annotated[Literal["confidential", "public"], PropertyInfo(alias="filter[client_type]")]
    """Filter by client type"""

    filter_name: Annotated[str, PropertyInfo(alias="filter[name]")]
    """Filter by exact client name"""

    filter_name_contains: Annotated[str, PropertyInfo(alias="filter[name][contains]")]
    """Filter by client name containing text"""

    filter_verified: Annotated[bool, PropertyInfo(alias="filter[verified]")]
    """Filter by verification status"""

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]
    """Page number"""

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]
    """Number of results per page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["TexmlSecretsResponse", "Data"]


class Data(BaseModel):
    name: Optional[str] = None

    value: Optional[Literal["REDACTED"]] = None


class TexmlSecretsResponse(BaseModel):
    data: Optional[Data] = None
